import 'dart:io';

import 'package:ricky_icon_generator/core/templates/templates.dart';
import 'package:image/image.dart';
import 'package:meta/meta.dart';
import 'package:ricky_icon_generator/core/base_controller.dart';

import '../../../core/constants.dart';

import '../../../core/models/icon_template_model.dart';
import '../../../utils/app_image_utils.dart';
import '../../../utils/general_utils.dart';
import '../base_app_icon_controller.dart';

class IOSAppIconController extends BaseAppIconController<IOSIconTemplateModel> {
  IOSAppIconController({required String backgroundColor, required Image customSourceImage, double? resizePercentage})
      : super(backgroundColor: backgroundColor, customSourceImage: customSourceImage, resizePercentage: resizePercentage);

  @experimental
  IOSAppIconController.custom(
      {required String backgroundColor,
        required Image customSourceImage,
        required double resizePercentage,
        ErrorHandler? errorHandler,
        String? rootPath})
      : super.custom(
      backgroundColor: backgroundColor,
      customSourceImage: customSourceImage,
      resizePercentage: resizePercentage,
      errorHandler: errorHandler,
      rootPath: rootPath);

  @override
  String get platform => kiOSPlatform;

  @override
  List<IOSIconTemplateModel> get appIconList => <IOSIconTemplateModel>[
    IOSIconTemplateModel(path: 'ic_launcher-20', size: 20.0),
    IOSIconTemplateModel(path: 'ic_launcher-20@2x', size: 40.0),
    IOSIconTemplateModel(path: 'ic_launcher-20@3x', size: 60.0),
    IOSIconTemplateModel(path: 'ic_launcher-29', size: 29.0),
    IOSIconTemplateModel(path: 'ic_launcher-29@2x', size: 58.0),
    IOSIconTemplateModel(path: 'ic_launcher-29@3x', size: 87.0),
    IOSIconTemplateModel(path: 'ic_launcher-40', size: 40.0),
    IOSIconTemplateModel(path: 'ic_launcher-40@2x', size: 80.0),
    IOSIconTemplateModel(path: 'ic_launcher-40@3x', size: 120.0),
    IOSIconTemplateModel(path: 'ic_launcher-60', size: 60.0),
    IOSIconTemplateModel(path: 'ic_launcher-60@2x', size: 120.0),
    IOSIconTemplateModel(path: 'ic_launcher-60@3x', size: 180.0),
    IOSIconTemplateModel(path: 'ic_launcher-76', size: 76.0),
    IOSIconTemplateModel(path: 'ic_launcher-76@2x', size: 156.0),
    IOSIconTemplateModel(path: 'ic_launcher-83.5@2x', size: 167.0),
    IOSIconTemplateModel(path: 'ic_launcher-1024', size: 1024.0),
  ];

  @override
  void executeConfigurationProcess() => _generateContentJson();

  @override
  void generateAppIcon() {
    logger('Generating app icons');

    // Create alpha layer for xxxhdpi foreground icon (the biggest dimension in android)
    final baseAlphaChannelLayerImage = Image((customSourceImage.width).toInt(), (customSourceImage.width).toInt())
      ..fillBackground(ColorUtils.hexToColor(backgroundColor!).value);

    final resizedBaseForegroundImage = copyResize(
      customSourceImage,
      width: (customSourceImage.width * resizePercentage).toInt(),
      height: (customSourceImage.height * resizePercentage).toInt(),
      interpolation: Interpolation.average,
    );

    // Define paddings for [resizedBaseForegroundImage]
    final paddingX = (baseAlphaChannelLayerImage.width - resizedBaseForegroundImage.width) ~/ 2;
    final paddingY = (baseAlphaChannelLayerImage.height - resizedBaseForegroundImage.height) ~/ 2;

    final foregroundImage = drawImage(baseAlphaChannelLayerImage, resizedBaseForegroundImage, dstX: paddingX, dstY: paddingY);

    for (var template in appIconList) {
      AppImageUtils.saveImage(resFolder: getFullPath(kiOSAppIconsImageFolder), template: template, image: foregroundImage);
    }
  }

  void _generateContentJson() {
    logger('Generating content.json file');

    final contentFile = File(getFullPath(kiOSContentFile));
    if (contentFile.existsSync()) {
      contentFile.delete(recursive: true);
    }
    contentFile
      ..createSync(recursive: true)
      ..writeAsStringSync(kiOSContentJson);
  }
}
